from typing import List
from simple_logger.logger import get_logger
from azure.mgmt.subscription import SubscriptionClient
from azure.mgmt.redhatopenshift import AzureRedHatOpenShiftClient


LOGGER = get_logger(name="microsoft-azure-utils")


def get_aro_supported_versions(aro_client: AzureRedHatOpenShiftClient, region: str) -> List[str]:
    supported_versions: List[str] = [
        aro_version.version for aro_version in aro_client.open_shift_versions.list(location=region)
    ]
    LOGGER.info(f"ARO supported versions: {supported_versions}")
    return supported_versions


def get_azure_supported_regions(subscription_client: SubscriptionClient, subscription_id: str) -> List[str]:
    supported_regions: List[str] = [
        region.name for region in subscription_client.subscriptions.list_locations(subscription_id=subscription_id)
    ]
    LOGGER.info(f"ARO supported regions: {supported_regions}")
    return supported_regions



from azure.identity import ClientSecretCredential
from azure.mgmt.subscription import SubscriptionClient
from azure.mgmt.network import NetworkManagementClient
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.redhatopenshift import AzureRedHatOpenShiftClient


def azure_credentials(tenant_id: str, client_id: str, client_secret: str) -> ClientSecretCredential:
    return ClientSecretCredential(
        tenant_id=tenant_id,
        client_id=client_id,
        client_secret=client_secret,
    )


def aro_client(credential: ClientSecretCredential, subscription_id: str) -> AzureRedHatOpenShiftClient:
    return AzureRedHatOpenShiftClient(credential=credential, subscription_id=subscription_id)


def network_client(credential: ClientSecretCredential, subscription_id: str) -> NetworkManagementClient:
    return NetworkManagementClient(credential=credential, subscription_id=subscription_id)


def resource_client(credential: ClientSecretCredential, subscription_id: str) -> ResourceManagementClient:
    return ResourceManagementClient(credential=credential, subscription_id=subscription_id)


def subscription_client(credential: ClientSecretCredential) -> SubscriptionClient:
    return SubscriptionClient(credential=credential)



from __future__ import annotations
import multiprocessing
import re
import shlex
from multiprocessing import Queue
from typing import List

from ocp_utilities.utils import run_command
from simple_logger.logger import get_logger

from clouds.aws.aws_utils import (
    delete_all_objects_from_s3_folder,
    delete_bucket,
)
from clouds.aws.session_clients import ec2_client, iam_client, rds_client, s3_client

LOGGER = get_logger(name="aws-nuke-cli")
MAX_ITEMS: int = 1000


def delete_rds_instances(region_name: str) -> bool:
    db_instances = rds_client(region_name=region_name).describe_db_instances()["DBInstances"]
    for db_instance_identifier in db_instances:
        LOGGER.warning(f"DB instance identifier: {db_instance_identifier}")
        # TODO: once we have the contents of db_instance_identifier, update code to delete it update return value
        # aws rds modify-db-instance --no-deletion-protection --db-instance-identifier "${rds}"

    return False


def delete_vpc_peering_connections(region_name: str) -> bool:
    for conn in ec2_client(region_name=region_name).describe_vpc_peering_connections()["VpcPeeringConnections"]:
        LOGGER.warning(f"VPC peering connection: {conn}")
        # TODO: once we have the contents of conn, update code to delete it update return value
        # aws ec2 delete-vpc-peering-connection --vpc-peering-connection-id "${pc}"

    return False


def delete_open_id_connect_providers(region_name: str) -> bool:
    LOGGER.info("Executing delete_open_id_connect_provider")
    _iam_client = iam_client(region_name=region_name)
    for conn in _iam_client.list_open_id_connect_providers()["OpenIDConnectProviderList"]:
        conn_name = conn["Arn"]
        LOGGER.info(f"Delete open id connection provider {conn_name}")
        _iam_client.delete_open_id_connect_provider(OpenIDConnectProviderArn=conn_name)

    return True


def delete_instance_profiles(region_name: str) -> bool:
    LOGGER.info("Executing delete_instance_profiles")
    _iam_client = iam_client(region_name=region_name)
    instance_profiles_dict = _iam_client.list_instance_profiles(MaxItems=MAX_ITEMS)
    for profile in instance_profiles_dict["InstanceProfiles"]:
        profile_name = profile["InstanceProfileName"]
        for role in _iam_client.get_instance_profile(InstanceProfileName=profile_name)["InstanceProfile"]["Roles"]:
            role_name = role["RoleName"]
            LOGGER.info(f"Remove role {role_name} from instance profile {profile_name}")
            _iam_client.remove_role_from_instance_profile(InstanceProfileName=profile_name, RoleName=role_name)
        LOGGER.info(f"Delete Profile {profile_name}")
        _iam_client.delete_instance_profile(InstanceProfileName=profile_name)

    return instance_profiles_dict["IsTruncated"]


def delete_roles(region_name: str) -> bool:
    LOGGER.info("Executing delete_roles")
    _iam_client = iam_client(region_name=region_name)
    roles_dict = _iam_client.list_roles(MaxItems=MAX_ITEMS)
    attached_role_policies_dict = {}
    detached_policy_names_dict = {}
    for role in roles_dict["Roles"]:
        role_name = role["RoleName"]
        if not re.search(
            r"ManagedOpenShift|AWS|OrganizationAccountAccessRole|pco-|RedHatIT|RH-",
            role_name,
        ):
            # Need to iterate over both list_attached_role_policies and list_role_policies
            # For attached policies, we need to detach them using ARN
            attached_role_policies_dict = _iam_client.list_attached_role_policies(RoleName=role_name, MaxItems=1000)
            for attached_policy in attached_role_policies_dict["AttachedPolicies"]:
                attached_policy_name = attached_policy["PolicyName"]
                LOGGER.info(f"Detach policy {attached_policy_name} for role {role_name}")
                _iam_client.detach_role_policy(RoleName=role_name, PolicyArn=attached_policy["PolicyArn"])

            # For detached policies, we need to delete them by name
            detached_policy_names_dict = _iam_client.list_role_policies(RoleName=role_name, MaxItems=1000)
            for detached_policy_name in detached_policy_names_dict["PolicyNames"]:
                LOGGER.info(f"Delete policy {detached_policy_name} for role {role_name}")
                _iam_client.delete_role_policy(RoleName=role_name, PolicyName=detached_policy_name)

            LOGGER.info(f"Delete role {role_name}")
            _iam_client.delete_role(RoleName=role_name)

    return any([
        roles_dict["IsTruncated"],
        attached_role_policies_dict.get("IsTruncated"),
        detached_policy_names_dict.get("IsTruncated"),
    ])


def delete_buckets(region_name: str) -> bool:
    LOGGER.info("Executing delete_buckets")
    _s3_client = s3_client(region_name=region_name)
    buckets_dict = _s3_client.list_buckets(MaxItems=MAX_ITEMS)
    for bucket in buckets_dict["Buckets"]:
        delete_all_objects_from_s3_folder(bucket_name=bucket["Name"], boto_client=_s3_client)
        delete_bucket(bucket_name=bucket["Name"], boto_client=_s3_client)

    return buckets_dict.get("IsTruncated")


def clean_aws_resources(aws_regions: List[str]) -> None:
    rerun_cleanup_regions_list = []
    jobs = []
    cleanup_queue: Queue[str] = multiprocessing.Queue()

    for region in aws_regions:
        job = multiprocessing.Process(
            name=f"--- Clean up {region} ---",
            target=clean_aws_region,
            kwargs={"aws_region": region, "queue": cleanup_queue},
        )
        jobs.append(job)
        job.start()

    while not cleanup_queue.empty():
        rerun_cleanup_regions_list.append(cleanup_queue.get())

    for _job in jobs:
        _job.join()

    if rerun_cleanup_regions_list:
        clean_aws_resources(aws_regions=rerun_cleanup_regions_list)


def clean_aws_region(aws_region: str, queue: Queue[str]) -> None:
    """Deletes AWS resources.

    Deletes open id connector providers, instance profiles and roles.
    (calls rds instances and vpc peerings connections - currently to only log data).
    Runs `cloud-nuke` utility - https://github.com/gruntwork-io/cloud-nuke
    Deletes S2 buckets.
    Adds aws_region to queue in case additional cleanup is needed.

    Args:
        aws_region (str): AWS region name
        queue (Queue): multiprocessing queue to pass result

    """
    LOGGER.info(f"Deleting resources in region {aws_region}")
    rerun_results: List[bool] = [
        delete_rds_instances(region_name=aws_region),
        delete_vpc_peering_connections(region_name=aws_region),
        delete_open_id_connect_providers(region_name=aws_region),
        delete_instance_profiles(region_name=aws_region),
        delete_roles(region_name=aws_region),
    ]
    run_command(command=shlex.split(f"cloud-nuke aws --region {aws_region} --force"))
    rerun_results.append(delete_buckets(region_name=aws_region))

    if any(rerun_results):
        queue.put(aws_region)



from typing import List
from simple_logger.logger import get_logger
from azure.core.exceptions import HttpResponseError
from clouds.microsoft_azure.session_clients import azure_credentials, resource_client

LOGGER = get_logger(name="azure-nuke-cli")


def nuke_all_azure_resources(tenant_id: str, client_id: str, client_secret: str, subscription_id: str) -> None:
    """
    Run nuke for all Azure cloud resources associated with the given credentials.
    This action is irreversible and will permanently delete all resources.

    Args:
        tenant_id (str): The Azure managed-identity tenant ID.
        client_id (str): The Azure managed-identity client ID.
        client_secret (str): The Azure managed-identity client secret.
        subscription_id (str): The Azure subscription ID.
    """
    _resource_client = resource_client(
        credential=azure_credentials(
            tenant_id=tenant_id,
            client_id=client_id,
            client_secret=client_secret,
        ),
        subscription_id=subscription_id,
    )

    failed_delete_resource_groups: List[str] = []
    azure_post_cleanup_message: str = "Azure cleanup completed"

    LOGGER.info("Starting Azure resources cleanup")
    for resource_group_name in [resource_group.name for resource_group in _resource_client.resource_groups.list()]:
        try:
            LOGGER.info(f"Deleting resource group {resource_group_name}")
            _resource_client.resource_groups.begin_delete(resource_group_name=resource_group_name)
        except HttpResponseError as ex:
            LOGGER.error(f"Failed to delete resource group {resource_group_name}: {ex}")
            failed_delete_resource_groups.append(resource_group_name)

    if failed_delete_resource_groups:
        LOGGER.error(
            f"{azure_post_cleanup_message} except for the following resource groups: {failed_delete_resource_groups}"
        )
    else:
        LOGGER.success(azure_post_cleanup_message)



from __future__ import annotations

from typing import Any

import boto3
import botocore


def aws_session(**kwargs: Any) -> boto3.session.Session:
    # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html#boto3.session.Session.client
    return boto3.session.Session(**kwargs)


def iam_client(**kwargs: Any) -> "botocore.client.IAM":
    return aws_session(**kwargs).client(service_name="iam")


def ec2_client(**kwargs: Any) -> "botocore.client.EC2":
    return aws_session(**kwargs).client(service_name="ec2")


def s3_client(**kwargs: Any) -> "botocore.client.S3":
    return aws_session(**kwargs).client(service_name="s3")


def rds_client(**kwargs: Any) -> "botocore.client.RDS":
    return aws_session(**kwargs).client(service_name="rds")



import os
import click
import shutil
from simple_logger.logger import get_logger
from clouds.cli.aws.aws_cli import clean_aws_resources
from pyhelper_utils.runners import function_runner_with_pdb
from clouds.cli.microsoft_azure.microsoft_azure_cli import nuke_all_azure_resources
from clouds.aws.aws_utils import set_and_verify_aws_credentials, aws_region_names

LOGGER = get_logger(name="cloud-cli")


class CloudCLIError(Exception):
    pass


@click.group()
@click.option(
    "--pdb",
    help="Drop to `ipdb` shell on exception",
    is_flag=True,
    show_default=True,
)
def cloud_cli(pdb: bool) -> None:
    # This command is only to group aws and azure sub commands
    pass


@cloud_cli.command()
@click.option(
    "--aws-regions",
    help="""
        \b
        Comma-separated string of AWS regions to delete resources from.
        """,
)
@click.option(
    "--all-aws-regions",
    help="""
        \b
        Run on all AWS regions.
        """,
    is_flag=True,
)
def aws_nuke(aws_regions: str, all_aws_regions: bool) -> None:
    """
    Nuke all AWS cloud resources in given/all regions
    """

    if not shutil.which("cloud-nuke"):
        raise CloudCLIError("cloud-nuke is not installed; install from: https://github.com/gruntwork-io/cloud-nuke")

    if all_aws_regions and aws_regions or not (all_aws_regions or aws_regions):
        raise CloudCLIError("Either pass --all-aws-regions or --aws-regions to run cleanup")

    _aws_regions = aws_region_names() if all_aws_regions else aws_regions.split(",")

    set_and_verify_aws_credentials(region_name=_aws_regions[0])

    clean_aws_resources(aws_regions=_aws_regions)


@cloud_cli.command()
@click.option(
    "--azure-tenant-id",
    help="Azure's managed identity tenant ID, needed for Azure API clients.",
    type=str,
    default=os.environ.get("AZURE_TENANT_ID"),
)
@click.option(
    "--azure-client-id",
    help="Azure's managed identity client ID, needed for Azure API clients.",
    type=str,
    default=os.environ.get("AZURE_CLIENT_ID"),
)
@click.option(
    "--azure-client-secret",
    help="Azure's managed identity client secret, needed for Azure API clients.",
    type=str,
    default=os.environ.get("AZURE_CLIENT_SECRET"),
)
@click.option(
    "--azure-subscription-id",
    help="Azure subscription ID, needed for Azure API clients.",
    type=str,
    default=os.environ.get("AZURE_SUBSCRIPTION_ID"),
)
def azure_nuke(
    azure_tenant_id: str, azure_client_id: str, azure_client_secret: str, azure_subscription_id: str
) -> None:
    """
    Nuke all Microsoft Azure cloud resources.
    """
    nuke_all_azure_resources(
        tenant_id=azure_tenant_id,
        client_id=azure_client_id,
        client_secret=azure_client_secret,
        subscription_id=azure_subscription_id,
    )


if __name__ == "__main__":
    function_runner_with_pdb(func=cloud_cli)



from typing import List

from google.cloud import compute_v1
from google.oauth2 import service_account


def get_gcp_regions(gcp_service_account_file: str) -> List[str]:
    credentials = service_account.Credentials.from_service_account_file(filename=gcp_service_account_file)
    return [
        region.name
        for region in compute_v1.RegionsClient(credentials=credentials).list(project=credentials.project_id).items
    ]



from __future__ import annotations
import os
import re
from typing import Any, List

import boto3
import botocore
import click
from simple_logger.logger import get_logger

from clouds.aws.aws_utils import delete_all_objects_from_s3_folder, delete_bucket

LOGGER = get_logger(name=__name__)


def delete_velero_cluster_buckets(cluster: str, boto_client: "botocore.client.S3") -> None:
    """
    Delete the velero bucket associated with a cluster

    Args:
        cluster (str): The cluster name
        boto_client (botocore.client.S3): aws boto client

    """

    LOGGER.info(f"Delete velero buckets for '{cluster}' cluster")

    velero_bucket_list = get_velero_buckets(boto_client=boto_client)
    if not velero_bucket_list:
        LOGGER.info("No velero buckets found")
        return

    for bucket in velero_bucket_list:
        bucket_name = bucket["Name"]
        if verify_cluster_matches_velero_infrastructure_name(
            boto_client=boto_client,
            cluster_name=cluster,
            bucket_name=bucket_name,
        ):
            delete_all_objects_from_s3_folder(bucket_name=bucket_name, boto_client=boto_client)
            delete_bucket(bucket_name=bucket_name, boto_client=boto_client)
            # A cluster is mapped to only one bucket, so don't check any remaining buckets
            return

    LOGGER.info("No buckets deleted")


def get_velero_buckets(boto_client: "botocore.client.S3") -> List[dict[str, Any]]:
    """
    Get a list of velero buckets

    Args:
        boto_client: AWS client

    Returns:
        list of velero buckets
    """
    LOGGER.info("Get a list of velero buckets")

    buckets = boto_client.list_buckets()["Buckets"]
    return [bucket for bucket in buckets if re.search("managed-velero-backups-", bucket["Name"])]


def get_velero_infrastructure_name(bucket_name: str, boto_client: "botocore.client.S3") -> str | None:
    """
    Get the velero bucket infrastructure name

    Args:
        bucket_name: The name of the cluster bucket to delete
        boto_client: AWS client

    Returns:
        str or None: velero infrastructure name if found else None
    """

    LOGGER.info(f"Get tags for bucket: {bucket_name}")
    bucket_tags = boto_client.get_bucket_tagging(Bucket=bucket_name)["TagSet"]

    for tag in bucket_tags:
        if tag["Key"] == "velero.io/infrastructureName":
            return tag["Value"]

    return None


def verify_cluster_matches_velero_infrastructure_name(
    boto_client: "botocore.client.S3",
    cluster_name: str,
    bucket_name: str,
) -> bool:
    """
    Get the velero bucket infrastructure name and compare it with the cluster

    Args:
        boto_client: AWS client
        cluster_name: The cluster name
        bucket_name: The bucket name

    Returns:
         bool: True if the cluster matches a velero infrastructure name, False otherwise
    """

    LOGGER.info(
        "Verify cluster matches a velero infrastructure name via its bucket tag:" f" {bucket_name}",
    )

    velero_infrastructure_name = get_velero_infrastructure_name(bucket_name=bucket_name, boto_client=boto_client)

    # Verify if the bucket is associated with the cluster
    if velero_infrastructure_name and re.search(
        rf"{cluster_name}(-\w+)?$",
        velero_infrastructure_name,
    ):
        LOGGER.info(
            f"Verified cluster '{cluster_name}' is associated with velero"
            f" infrastructure name {velero_infrastructure_name}",
        )
        return True

    return False


@click.command()
@click.option(
    "--aws-access-key-id",
    help=("Set AWS access key id, default if taken from environment variable:" " AWS_ACCESS_KEY_ID"),
    required=True,
    default=os.getenv("AWS_ACCESS_KEY_ID"),
)
@click.option(
    "--aws-secret-access-key",
    help=("Set AWS secret access key id,default if taken from environment variable:" " AWS_SECRET_ACCESS_KEY"),
    required=True,
    default=os.getenv("AWS_SECRET_ACCESS_KEY"),
)
@click.option(
    "-c",
    "--cluster-name",
    help="",
    required=True,
    default=os.getenv("AWS_SECRET_ACCESS_KEY"),
)
def main(aws_access_key_id: str, aws_secret_access_key: str, cluster_name: str) -> None:
    boto_client = boto3.client(
        service_name="s3",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    delete_velero_cluster_buckets(cluster=cluster_name, boto_client=boto_client)


if __name__ == "__main__":
    main()



from __future__ import annotations
import json
import os
from configparser import ConfigParser, NoOptionError, NoSectionError
from http import HTTPStatus
from typing import List, Optional

import botocore
from simple_logger.logger import get_logger

from clouds.aws.session_clients import ec2_client

LOGGER = get_logger(name=__name__)
AWS_CONFIG_FILE: str = os.environ.get("AWS_CONFIG_FILE", os.path.expanduser("~/.aws/config"))
AWS_CREDENTIALS_FILE: str = os.environ.get("AWS_SHARED_CREDENTIALS_FILE", os.path.expanduser("~/.aws/credentials"))


class AWSConfigurationError(Exception):
    pass


def set_and_verify_existing_config_in_env_vars_or_file(
    vars_list: List[str], file_path: str, section: str = "default"
) -> None:
    """
    Verify vars are either set as environment variables or in a config file.

    When var exists as OS environment then continue.
    When vars exists in config file, set them as OS environment.
    If none of the above raise.

    Args:
        vars_list (list): A list of variable names
        file_path (str): Absolute path to config file
        section (str): Section name in config file

    Raises:
        AWSConfigurationError: raises on missing configuration
    """
    missing_vars = []
    parser = ConfigParser()
    parser.read(file_path)
    aws_region_str = "region"
    for var in vars_list:
        if os.getenv(var.upper()):
            continue

        LOGGER.info(f"Variable {var} is not set as environment variables, checking in config" " file.")
        try:
            var_in_file = var.lower()
            # `AWS_REGION` environment variable is set as `region` in the config file
            if aws_region_str in var_in_file:
                var_in_file = aws_region_str

            os.environ[var.upper()] = parser.get(section, var_in_file)
        except (NoSectionError, NoOptionError):
            missing_vars.append(var)

    if missing_vars:
        LOGGER.error(
            f"Missing configuration: {','.join(missing_vars)}. "
            f"Values should be set in environment variables or in {file_path}"
        )
        raise AWSConfigurationError()


def set_and_verify_aws_credentials(region_name: Optional[str | None]) -> None:
    set_and_verify_existing_config_in_env_vars_or_file(
        vars_list=["AWS_ACCESS_KEY_ID", "AWS_SECRET_ACCESS_KEY"],
        file_path=AWS_CREDENTIALS_FILE,
    )

    ec2_client(region_name=region_name).describe_regions()


def set_and_verify_aws_config() -> None:
    set_and_verify_existing_config_in_env_vars_or_file(
        vars_list=["AWS_REGION"],
        file_path=AWS_CONFIG_FILE,
    )


def delete_all_objects_from_s3_folder(bucket_name: str, boto_client: "botocore.client.S3") -> None:
    """
    Deletes all files in a folder of an S3 bucket

    Args:
        bucket_name: The bucket name
        boto_client: AWS S3 client
    """
    try:
        objects = boto_client.list_objects_v2(Bucket=bucket_name)
    except boto_client.exceptions.NoSuchBucket:
        LOGGER.warning(f"{bucket_name} not found")
        return

    #  Sometimes there maybe no contents -- no objects
    files_in_folder = objects.get("Contents")
    if not files_in_folder:
        LOGGER.info(f"No objects to be deleted for {bucket_name}")
        return

    files_to_delete = [{"Key": file["Key"]} for file in files_in_folder]
    delete_response = boto_client.delete_objects(
        Bucket=bucket_name,
        Delete={"Objects": files_to_delete, "Quiet": True},
    )

    delete_response_http_status_code = delete_response["ResponseMetadata"]["HTTPStatusCode"]
    if delete_response_http_status_code == HTTPStatus.OK:
        LOGGER.info("Objects deleted successfully")
        return
    else:
        LOGGER.error(
            "Objects not deleted:\n:" f"{json.dumps(delete_response, default=str, indent=4)}",
        )


def delete_bucket(bucket_name: str, boto_client: "botocore.client.S3") -> None:
    """
    Delete velero bucket

    Args:
        bucket_name: the bucket name
        boto_client: AWS client
    """
    LOGGER.info(f"Delete bucket {bucket_name}")
    try:
        response = boto_client.delete_bucket(Bucket=bucket_name)
    except boto_client.exceptions.NoSuchBucket:
        LOGGER.warning(f"{bucket_name} not found")
        return

    response_http_status_code = response["ResponseMetadata"]["HTTPStatusCode"]
    if response_http_status_code == HTTPStatus.NO_CONTENT:
        LOGGER.info(f"Cluster bucket '{bucket_name}' deleted successfully")
        return

    elif response_http_status_code == HTTPStatus.NOT_FOUND:
        LOGGER.info(f"Bucket '{bucket_name}' not found")
        return

    LOGGER.error(
        f"Bucket {bucket_name} not deleted",
        json.dumps(response, defualt=str, indent=4),
    )


def aws_region_names() -> List[str]:
    """
    Lists AWS regions

    Returns:
        list: list of AWS regions name

    """
    regions = ec2_client().describe_regions()["Regions"]
    return [region["RegionName"] for region in regions]


def get_least_crowded_aws_vpc_region(region_list: List[str]) -> str:
    """
    Selects region with the least number of VPCs.

    Args:
        region_list: list of region names

    Returns:
        str: region name with have the least number of VPCs

    """
    if not region_list:
        raise ValueError("`region_list` must not be empty")

    region, vpcs = region_list[0], 0
    for _region in region_list:
        if (num_vpcs := len(ec2_client(region_name=_region).describe_vpcs()["Vpcs"])) <= vpcs:
            region = _region
            vpcs = num_vpcs
    return region



from typing import Any, Dict, List

import boto3
import botocore
from simple_logger.logger import get_logger

LOGGER = get_logger(name=__name__)
DEFAULT_AWS_REGION: str = "us-east-1"


def iam_client(region: str = DEFAULT_AWS_REGION) -> "botocore.client.IAM":
    """Creates an IAM client.

    Args:
        region (str, default: "us-east-1"): Region to use for session, a client is associated with a single region.

    Returns:
        botocore.client.IAM: Service client instance.

    """
    LOGGER.info(f"Creating IAM client using region {region}.")
    return boto3.client(service_name="iam", region_name=region)


def get_roles(client: "botocore.client.IAM" = None) -> List[Dict[str, Any]]:
    """
    Get all IAM roles.

    Args:
        client (botocore.client.IAM, optional): A boto3 client for IAM. Defaults to None.

    Returns:
        List[Dict[str, Any]]: A list of IAM roles
    """
    LOGGER.info("Retrieving all roles from IAM.")

    iam_max_items = 1000
    client = client or iam_client()
    response = client.list_roles(MaxItems=iam_max_items)
    roles = response["Roles"]

    while response["IsTruncated"]:
        marker = response["Marker"]
        response = client.list_roles(Marker=marker, MaxItems=iam_max_items)
        roles.extend(response["Roles"])

    return roles


def create_or_update_role_policy(
    role_name: str,
    policy_name: str,
    policy_document: str,
    region: str = DEFAULT_AWS_REGION,
) -> None:
    """
    Create a new policy role or update an existing one.

    Args:
        role_name (str): role policy name
        policy_name (str): policy name
        policy_document (str): policy documents as Json file content
        region (str): aws region
    """
    client = iam_client(region=region)
    LOGGER.info(f"Create/Update role {role_name} for policy {policy_name} by documented policy.")
    client.put_role_policy(
        RoleName=role_name,
        PolicyName=policy_name,
        PolicyDocument=policy_document,
    )
